#! /usr/bin/env python

'''Parallel shell based on SSS PM interface'''
__revision__ = '$Revision: 1.15 $'

import atexit
from getopt import getopt, GetoptError
from elementtree.ElementTree import Element, XML, SubElement, tostring
from os import environ, getuid
from pwd import getpwuid
from sss.ssslib import comm_lib, LookupError as SSSLookupError
from sss.server import EventReceiver
from sys import argv

class PMEvent(EventReceiver):
    '''PMEvent waits until process group has completed'''
    def __init__(self, pgrp, dbg=0):
        self.dispatch = {'event':self.HandleXMLEvent_cb}
        EventReceiver.__init__(self, debug=dbg)
        self.__subscriptions__ = [('process-manager', '**', pgrp)]

    def HandleXMLEvent_cb(self, xem, portinfo):
        '''Shutdown when process_end event arrives'''
        for event in xem.findall('.//event'):
            if event.get("msg") == "process_end":
                self.shut = 1
        return Element('event-ok')
    
if __name__ == '__main__':
    try:
        (opts, args) = getopt(argv[1:], 'aw:g:f:d')
    except GetoptError, msg:
        print "Usage: dsh.py [-a] [-d] [-g group] [-f file] [-w node1,node2] cmd"
        print msg
        raise SystemExit, 1

    mode = None
    for opt in opts:
        if opt[0] == '-a':
            mode = 'all'
        elif opt[0] == '-g':
            mode = 'group'
            tag = opt[1]
        elif opt[0] == '-f':
            mode = 'node'
            nodes = [item.strip() for item in open(opt[1]).readlines()]
        elif opt[0] == '-w':
            mode = 'node'
            nodes = opt[1].split(',')
        elif opt[0] == '-d':
            pass
        else:
            print "semantic error"
            raise SystemExit, 1

    if not args:
        print "Usage: dsh.py [-a] [-d] [-g group] [-f file] [-w node1,node2] cmd"
        print "One of -a, -g, -f, or -w are required"
        raise SystemExit, 1

    cmd = args[0]

    args = args[1:]

    if not mode:
        print "Usage: dsh.py [-a] [-d] [-g group] [-f file] [-w node1,node2] cmd"
        print "One of -a, -g, -f, or -w are required"
        raise SystemExit, 1

    submitter = getpwuid(getuid())[0]
    output = 'display'
    path = environ['PATH']
    cwd = environ['PWD']

    comm = comm_lib(debug = ('-d' in argv))

    #if mode=group, query the build-system to find out the list of nodes
    if mode == 'group':
        query = Element('get-tag', node='*')
        bsh = comm.ClientInit('build-system')
        comm.SendMessage(bsh, tostring(query))
        answer = comm.RecvMessage(bsh)
        comm.ClientClose(bsh)
        data = XML(answer)
        # not finished here but I am waiting to test it for real on my machine, wanted to checkin 
        # put the hostlist into nodes
        nodes = [tag.get('node') for tag in XML.findall('tag')]
    
    # at this point, regardless of mode, nodes is populated
    numprocs = str(len(nodes))

    pmcmd = Element("create-process-group", totalprocs=numprocs, submitter=submitter,
                    output=output)
    ps = SubElement(pmcmd, "process-spec", path=path, cwd=cwd)
    ps.attrib['exec'] = cmd
    hs = SubElement(pmcmd)
    hs.text = ('\n').join(nodes.join)
    for arg in range(len(args)):
        SubElement(ps, 'arg', idx=str(arg), value=args[arg])
        
    try:
        pm = comm.ClientInit('process-manager')
    except SSSLookupError:
        print "Process Manager down"
        raise SystemExit, 1
        
    comm.SendMessage(pm, tostring(pmcmd))
    ack = comm.RecvMessage(pm)
    comm.ClientClose(pm)

    xml = XML(ack)
    if xml.tag == 'error':
        print ack
        raise SystemExit, 1
    pgid = xml.get('pgid')
    debug = argv.count('-d')

    server = PMEvent(pgid, debug = debug)
    server.ServeForever()
    server.Shutdown()
    atexit._exithandlers = []
    del server
    # Now the process is done; we need to collect output
    pm = comm.ClientInit('process-manager')
    comm.SendMessage(pm, "<get-process-group-info><process-group pgid='%s'/></get-process-group-info>" % pgid)
    resp = comm.RecvMessage(pm)
    comm.ClientClose(pm)
    rxml = XML(resp)

    output = ''
    err = ''
    try:
        output = "".join([node.text for node in rxml.findall('.//output')])
    except:
        pass

    try:
        err = rxml.findall('.//error').text
    except:
        pass

    if output:
        print output,
    if err:
        print err,
        raise SystemExit, 1
