#!/usr/bin/env python

'''Event Manager query tool'''
__revision__ = '$Revision: 1.11 $'

from getopt import getopt, GetoptError
from sys import argv
from elementtree.ElementTree import XML, tostring

from sss.ssslib import comm_lib, LookupError as SSSLookupError, ProtocolError

if __name__ == '__main__':
    try:
        (opts, args) = getopt(argv[1:], 'esdc:m:n:a:')
    except GetoptError, m:
        print m
        print "Usage: eminfo.py [-e] [-s] [-d] [-c component] [-m msg] [-a data] [-n notify]"
        raise SystemExit, 1

    component = '*'
    msg = '*'
    data = '*'
    notify = '*'

    for opt in opts:
        if opt[0] == '-c':
            component = opt[1]
        elif opt[0] == '-m':
            msg = opt[1]
        elif opt[0] == '-n':
            notify = opt[1]
        elif opt[0] == '-a':
            data = opt[1]

    if '-s' in argv:
        emsg = '''<get-subscription><subscription component='%s' msg='%s' data='%s' notify='%s' matches='*' delivered='*' failures='*' last='*'/></get-subscription>''' % (component, data, msg, notify)
    elif '-e' in argv:
        emsg = "<get-event><event component='%s' msg='%s' data='%s' time='*'/></get-event>" % (component, msg, data)
    else:
        print "Usage: eminfo.py [-e] [-s] [-d] [-c component]"
        print "one of -e or -s are required"
        raise SystemExit, 1

    debug = len([x for x in argv if x=='-d'])
    c = comm_lib(debug=debug)
    try:
        h = c.ClientInit("event-manager")
    except SSSLookupError:
        print "Event Manager not registered"
        raise SystemExit, 1
    except ProtocolError:
        print "Protocol error"
        raise SystemExit, 1
    c.SendMessage(h, emsg)
    resp = c.RecvMessage(h)
    c.ClientClose(h)
    element = XML(resp)
    if element.tag == 'Error':
        print "Server failure"
    else:
        for item in element.getchildren():
            print tostring(item)
