#! /usr/bin/env python

'''Interactive job-run command'''
__revision__ = '$Revision: 1.17 $'

from elementtree.ElementTree import Element, SubElement, XML, tostring
from os import environ, getuid
from pwd import getpwuid
from sss.ssslib import comm_lib
from sss.server import EventReceiver
from sys import argv

class procevent(EventReceiver):
    '''Sleep until process exit'''
    def __setup__(self):
        self.__subscriptions__ = [('process-manager', 'process_end', self.kwargs['pgid'])]

    def HandleEvent(self, xel, portinfo):
        for event in xel.getchildren():
            if event.attrib['msg'] == 'process_end':
                self.shut = 1
                print tostring(event)
        return Element("event-ok")
    
if __name__ == '__main__':
    executable = argv[1]
    numprocs   = argv[2]
    # submitter = environ['USER']
    submitter = getpwuid(getuid())[0]
    path = environ['PATH']
    cwd = environ['PWD']

    msg = Element("create-process-group", pgid='*', submitter=submitter, totalprocs=numprocs, output='merged')
    ps = SubElement(msg, "process-spec", user=submitter, path=path, cwd=cwd)
    ps.attrib['exec'] = executable

    print tostring(msg)
    
    comm = comm_lib(debug='-d' in argv)
    process_manager = comm.ClientInit('process-manager')
    comm.SendMessage(process_manager, tostring(msg))
    ack = comm.RecvMessage(process_manager)
    comm.ClientClose(process_manager)
    print ack
    r = XML(ack)
    pgid = r.attrib.get('pgid', None)

    if not pgid:
        raise SystemExit, 1
    else:
        print "Got PGID %s\nwaiting for event data" % (pgid)

    debug = argv.count('-d')
    server = procevent(pgid=pgid, debug=debug)
    server.ServeForever()

    # Now we pm-wait
    msg = XML("<wait-process-group><process-group pgid='%s' submitter='*'><output/><exit-status><exit-code host='*'/></exit-status></process-group></wait-process-group>" % (pgid))
    pm = comm.ClientInit('process-manager')
    comm.SendMessage(pm, tostring(msg))
    ack = comm.RecvMessage(pm)
    comm.ClientClose(pm)

    print ack
