#!/usr/bin/env python

from os import system, path, fork, close
from sys import exit
from signal import signal, SIGINT
from select import select
from string import split,strip
from syslog import syslog, LOG_INFO, LOG_ERR
from cPickle import dumps, loads
from elementtree.ElementTree import Element, SubElement, tostring
from time import strftime, localtime


from sss.server import Server, BufferedPipe
from sss.ssslib import comm_lib
from sss.restriction import DataSet, Data

class nsm(Server):
   __component__='node-state-manager'
   __implementation__='nsm'
   __statefields__ = ['nsdata']
   __dispatch__= {   'get-node-state':'nsdata.Get',
                     'set-node-state':'XSet',
                     'add-node-state':'nsdata.Add',
                     'del-node-state':'nsdata.Del',
                     'diagnose':'Diagnose'}
   __validate__ = 1


   def __setup__(self):
      self.nsdata = DataSet('node-states','node-state', Data, None, False)
      self.readConf('/etc/nsm.conf')

   def readConf(self,filename):
      try:
         for data in open(filename,'r').readlines():
            (var,value)=split(data,'=')
            setattr(self,var,strip(value))
      except IOError, e:
         print "Unable to open the nsm.conf file: ", e

   def XSet(self,xml,(peer,port)):
      self.changed_nodes = {}
      result = self.nsdata.Get(xml,(peer,port), self.Set)
      self.evt.WriteMessage(dumps(self.changed_nodes.keys()))
      return result

   def Set(self, item, attrib):
      for x in attrib.keys():
         if item.attr[x] != attrib[x]:
            item.attr[x]=attrib[x]
            self.changed_nodes[item.attr['node']]=1

   def Diagnose(self,xml,(peer,port)):
      level = xml.attrib['level']
      mode = xml.attrib['mode']
      nodexml = self.nsdata.Get(xml,(peer,port))
      nodelist = [nx.attrib['node'] for nx in nodexml]
      noderes = {}
      for node in nodelist:
         noderes[node] = self.runTest(node,level,mode)
      root = Element('set-node-state')
      for node in noderes.keys():
         if noderes[node]:
            as='online'
         else:
            as='offline'
         child = Element('node-state', node=node, adminstate=as)
         root.append(child)

      return XSet(root,(peer,port))


   def runTest(self,node,level,mode):
      #run the test, generate state based on exit code, update statedb, send event
      rc = system("%s %s %s %s"%(self.runcommand,node,level,mode))
      syslog(LOG_INFO, "%s %s %s %s"%(self.runcommand,node,level,mode))
      #the state may also be dependent on the level of the test that was run.
      if( rc != 0 ):
         return False
      else:
         return True
            
   def ChildSigHand(self,signum,frame):
      exit(0)
    
   def __setupChildren__(self):
      self.evt = BufferedPipe()
      self.pipes.append(self.evt)
      pid=fork()
      if pid==0:
         signal(SIGINT, self.ChildSigHand)
         self.readsockets=[self.evt.r]
         self.socketdispatch={self.evt.r:self.ProcessChanges}
         self.ServeForever()
      else:
         close(self.evt.r)
         return [pid]

   def ProcessChanges(self,pipe,inl=None):
      while 1:
         try:
            (infd,out,x)=select([self.evt.r],[],[],5)
         # Catch and disregard the traceback on child process exit
         except:
            pass
         if infd != []:
            self.evt.ReadMessages()

         while self.evt.buffer:
            changed_nodes=loads(self.evt.ReadMessage())
      
            # Send state changed message
            doc=Element('add-event')
            for node in changed_nodes:
               SubElement(doc, 'event',component=self.__component__, 
                          msg='state-changed', data=node, 
                          time=strftime("%c",localtime()))
            if len(doc) > 0:
                try:
                   c=comm_lib()
                   h=c.ClientInit('event-manager')
                   c.SendMessage(h,tostring(doc))
                   c.RecvMessage(h)
                   c.ClientClose(h)
                except:
                   syslog(LOG_ERR, "Failed to send state-changed event")
      
                del doc
      return 0
        

if __name__ == '__main__':
    from getopt import getopt, GetoptError
    from os import fork
    from sys import argv,exit
    try:
        (opts,args)=getopt(argv[1:],'d',['daemon='])
    except GetoptError,msg:
        print "%s\nUsage:\nnsm.py [-d] [--daemon <pidfile>]"%(msg)
        exit(1)
    daemon=filter(lambda x:(x[0] == '--daemon'),opts)
    ldebug=len((filter(lambda x:(x[0] == '-d'),opts)))
    if daemon:
        from sss.daemonize import daemonize
        daemonize(daemon[0][1])
    server=nsm(debug=ldebug)
    server.ServeForever()
